// Copyright (c) 2014 Tel-Aviv University (Israel).
// All rights reserved.
//
// This file is part of ACG (www.acg.cs.tau.ac.il).
// You can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Author(s): Michal Kleinbort <balasmic@post.tau.ac.il>
//            Oren Salzman <orenzalz@post.tau.ac.il >          

#ifndef RANDOM_UTILS_H__RTG
#define RANDOM_UTILS_H__RTG

#include <boost/random/uniform_real_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>

namespace RTG
{
    /// A class used for generating random d dimensional points.
    template <typename Point_d>
    class Random_utils
    {
     public:
        typedef boost::random::mt19937                              MTEng;
        typedef boost::random::uniform_real_distribution<double>    UniformDist;
        
     public: 
        
        /// \brief Constructor 
	    Random_utils(int seed = 1, double min=-1, double max = 1):
		    m_seed(seed), m_eng(seed), m_min(min), m_max(max), m_dist(min,max) 
        {};

     
        /// \brief returning a random real number according to a uniform distribution
        double get_random_num() const
        {
	        double n(m_dist(m_eng));
	        return n;
        }

  
        /// \brief returning a random point in dim dimensions.
        //         The value for each coordinate is chosen randomly
        //           according to a uniform distribution
        Point_d get_random_point(int dim) const
	    {
		    std::vector<double> coords;
		    for (int i=0; i < dim; ++i)
			    coords.push_back(get_random_num());
		    return Point_d(dim, coords.begin(), coords.end());
	    }

        //geometric random functions
        /// \brief updating the given vec with dim random elements
        //          chosen according to a uniform distribution
	    void  get_random_vec(int dim, std::vector<double>& vec) const
	    {
		    for (int i=0; i < dim; ++i)
			    vec.push_back(get_random_num());
	    }

     private:
	    int     m_seed;
	    double  m_max;
	    double  m_min;
	    mutable MTEng   m_eng;
	    mutable UniformDist m_dist;

	
    }; //Random_utils

} //namespace RTG

#endif //RANDOM_UTILS_H__RTG